#include "FurShader.h"
#include "gtc/type_ptr.hpp"

using namespace glm;

FurShader::FurShader(int density, int layers, float length, float thickness, vec3 grav, vec3 amb_col, vec3 diff_col)
: Shader("./Shader/FurShader.vert", "./Shader/FurShader.frag")
, fur_density(density)
, fur_layers(layers)
, fur_length(length)
, fur_thickness(thickness)
, fur_gravity(grav)
//, texture(new Texture(256, density)) // ("./Textures/noise.png")
{
	// generate textures with density according to layer
	for (auto lid = 0; lid < layers; lid++) {
		float l = (float) (lid + 1) / layers;
		textures.push_back(new Texture(256, diff_col, min((int)(fur_density * 3 * (1 - l)), (int)(256 * 256)), 23234));
	}	

	// init active texture
	if (textures.size() > 0) {
		texture = textures[0];
	}

	// at least 1 layer
	else {
		texture = new Texture(256, diff_col, fur_density, 23234);
		textures.push_back(texture);
	}

	useShader();

	// set material specific uniforms
	auto amb_col_location = glGetUniformLocation(programHandle, "material.ambient_color");
	auto diff_col_location = glGetUniformLocation(programHandle, "material.diffuse_color");
	glUniform3fv(amb_col_location, 1, glm::value_ptr(amb_col));
	glUniform3fv(diff_col_location, 1, glm::value_ptr(diff_col));

	// set fur specific uniforms
	auto thickness_location = glGetUniformLocation(programHandle, "fur.thickness");
	auto gravity_location = glGetUniformLocation(programHandle, "fur.gravity");
	
	glUniform1f(thickness_location, thickness);
	glUniform3fv(gravity_location, 1, glm::value_ptr(grav));
	

	glUseProgram(0);

}

FurShader::~FurShader() {
	delete texture;
	for (Texture* texture : textures) {
		delete texture; texture = nullptr;
	}	
}

void FurShader::bindTexture(int unit) {
	texture->bind(unit);
	if (useMipMapping == 0){ //use no mipmapping
		if (useLinearTexSampling){
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		}
		else{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		}
	}
	else if (useMipMapping == 1){ //use nearest neighbour mipmapping
		if (useLinearTexSampling){
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
		}
		else{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);
		}
	}
	else{
		if (useLinearTexSampling){ //use linear sampling mipmapping
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		}
		else{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
		}
	}
	auto tex_location = glGetUniformLocation(programHandle, "fur_texture");
	glUniform1i(tex_location, unit);
}

void FurShader::setLighting(std::vector<std::shared_ptr<PointLight>> *allLights, int sm_light_id) {
	useShader();

	glm::vec3 att[20];
	glm::vec3 intensity[20];
	glm::vec3 location[20];
	bool on[20];

	for (int i = 0; i < allLights->size(); i++){
		att[i] = vec3(allLights->at(i)->const_attenuation, allLights->at(i)->lin_attenuation, allLights->at(i)->quad_attenuation);
		location[i] = allLights->at(i)->location();
		intensity[i] = allLights->at(i)->intensity;// *vec3(1, 0.8, 0.1);
		on[i] = allLights->at(i)->on;
	}
	for (int i = allLights->size(); i < 20; i++){
		att[i] = vec3(0, 0, 0);
		location[i] = vec3(0, 0, 0);
		intensity[i] = vec3(0, 0, 0);
		on[i] = false;
	}

	auto light_count_location = glGetUniformLocation(programHandle, "light_count");
	auto light_loc_location = glGetUniformLocation(programHandle, "light_location");
	auto light_int_location = glGetUniformLocation(programHandle, "light_intensity");
	auto light_att_location = glGetUniformLocation(programHandle, "light_attenuation");
	auto light_on_location = glGetUniformLocation(programHandle, "light_on");
	glUniform1i(light_count_location, allLights->size());
	glUniform3fv(light_loc_location, 20, glm::value_ptr(location[0]));
	glUniform3fv(light_int_location, 20, glm::value_ptr(intensity[0]));
	glUniform3fv(light_att_location, 20, glm::value_ptr(att[0]));
	glUniform1iv(light_on_location, 20, (GLint*)&on[0]);

	// Shadow map uniforms
	auto id_loc = glGetUniformLocation(programHandle, "sm_light_id");
	auto pos_loc = glGetUniformLocation(programHandle, "sm_light_pos");
	glUniform1i(id_loc, sm_light_id);
	glUniform3fv(pos_loc, 1, glm::value_ptr(location[sm_light_id]));

	glUseProgram(0);
}

int FurShader::drawLayers() {
	return fur_layers;
}

void FurShader::setLayerUniforms(float layer_id) {

	// get texture for current layer
	if (layer_id > 0) {
		texture = textures[layer_id - 1];
	}
	// ignore texture for 0th layer
	else {
		texture = textures[0];
	}
	
	// calculate ratio
	float layer = float(layer_id) / fur_layers;

	// calculate length and gravity according to layer
	float length = fur_length * layer;
	//glm::vec3 gravity = fur_gravity;// -glm::vec3(0, 0.1 * layer, 0);

	// set uniforms
	auto length_location = glGetUniformLocation(programHandle, "fur.length");
	auto layer_location = glGetUniformLocation(programHandle, "fur.layer");
	//auto gravity_location = glGetUniformLocation(programHandle, "fur.gravity");
	glUniform1f(length_location, length);
	glUniform1f(layer_location, layer);
	//glUniform3fv(gravity_location, 1, glm::value_ptr(gravity));
}
