#version 400 core

in vec3 worldNormal;
in vec4 worldPos4D;
in vec3 viewVector;
in vec2 tex_coords;

layout(location = 0) out vec4 outColor;

uniform bool lightning_on;
uniform int light_count;
uniform vec3 light_location[20]; //Note: not more than 20 lights are allowed
uniform vec3 light_intensity[20];
uniform vec3 light_attenuation[20];
uniform ivec3 light_on[25]; //bool

uniform sampler2D fur_texture;

struct Fur {
	float layer;
	float length;
	float thickness;
	vec3 gravity;
};
uniform Fur fur;

struct Material {
	vec3 diffuse_color;
	vec3 ambient_color;
	vec3 specular_color;
	float shininess;
	float transparency;
};
uniform Material material;

uniform samplerCubeShadow shadow_map;  // CUBE shadow mapping
uniform int sm_light_id;
uniform vec3 sm_light_pos;
uniform vec2 planes;

void main()
{
		vec4 fur_color = texture(fur_texture,tex_coords);

		vec3 worldPos = worldPos4D.xyz;
	
		// Apply illumination model
		outColor = vec4(0,0,0,0);
		vec3 diffuse_color; 
		float alpha;

		// fur
		if (fur.layer > 0) {
			diffuse_color = fur_color.xyz *max(fur.layer,0.9);
			alpha = fur_color.a* (1-fur.layer);
		} 
		
		// skin
		else {
			diffuse_color = material.diffuse_color;
			alpha = 1;
		}
		 
		vec3 ambient_color = material.ambient_color;
		normalize(viewVector);
		normalize(worldNormal);

		// Shadow mapping scaling factor
		float is_lit = 1.0;
		float map_dist = 0.0;

		for(int i = 0; i < light_count; i++){
	
			vec3 light_dir = light_location[i]-worldPos;
			vec3 L = normalize(light_dir); // Vector from position(fragment) to light source
			vec3 H = normalize(L+viewVector);
		
			float d = length(light_dir);									// Distance between pos and point light
			float a = light_attenuation[i].x + d*light_attenuation[i].y + d*d*light_attenuation[i].z;// Attenuation
			if (light_on[i].x>0) { 
				a *= 0.1;
			}	
			vec3 I = light_intensity[i] / a;													// Intensity

			// Only use light source if player is close enough
			if (light_on[i].x<1) {
				 I *= 0.35;//continue; //I *= 10; hide object until close enough
			}
			I *= 5;

			if (lightning_on) {
				I *= 20;
			}

			float NdotL = max(dot(worldNormal,L),0.0);
			float NdotH = max(dot(worldNormal,H),0.0);
	
			vec3 color = diffuse_color*NdotL; // + specular_color*NdotL*pow(NdotH,shininess);

			// Perform CUBE shadow mapping for one light source
			if (i==sm_light_id) {
				float d_norm = (d-planes.x)/(planes.y - planes.x);
				d_norm = clamp(d_norm, 0, 1);
				//normalize(light_dir);
				is_lit = texture(shadow_map, vec4(-light_dir, d/planes.y)) ;
				//is_lit = (d_norm>map_dist+0.05) ? 0.5 : 1.0;
				color *= is_lit;
			}
			
			vec3 illumination = I*(color);
		
			outColor = outColor + vec4(illumination, 1.0);
		}
		

		// Add ambient light
		outColor = outColor + vec4(ambient_color, 1.0);

		// Desaturate
		if (lightning_on) {
				outColor = vec4(vec3(length(vec3(outColor.x, outColor.y, outColor.z))), 1.0);
		}

		// Scale outColor to [0,1]
		if (max(outColor.x, max(outColor.y, outColor.z))>1) {
			outColor = outColor * 1.0f/max(outColor.x, max(outColor.y, outColor.z));
		}

		// Transparency
		outColor = vec4(outColor.xyz, alpha);
		//outColor = vec4(outColor.xyz, fur_color.a);//;
		
}		

